/**
 * Creates a stickit + multiselect filter combo.
 *
 * Extend and define:
 *
 * required to fill in
 * - createSourceCollection
 * - filterKey
 * - ui.selectStickit
 *
 * optional:
 * - multiselectOptions
 * - multiselectFilterOptions
 */
export default Marionette.Behavior.extend({
    filterKey: '',
    cSource: null,
    initialize() {
        this.cSource = this.createSourceCollection(this.view);
        this.setDefaultAttributes(this.cSource);
        this.refetch();

        this.listenTo(this.cSource, 'refetch', this.refetch);
    },
    createSourceCollection(view) {

    },
    getTargetModel(view) {
        return view.collection.attributes;
    },
    fetch() {
        // Set silent: true, so that backbone.stickit won't rerender the select
        // on each add. Afterwards we set trigger a fake reset so stickit will
        // pick up the change.
        return this.cSource.fetch({ silent: true });
    },
    afterFetch() {
        this.setInitialState();
        this.cSource.trigger('reset');
        this.ui.selectStickit.multiselect('refresh');
    },
    refetch() {
        const xhr = this.fetch();

        if (xhr && xhr.done) {
            xhr.done(() => this.afterFetch());
        } else {
            this.afterFetch();
        }
    },
    setDefaultAttributes(cSource) {
        cSource.attributes.set('limit', 'none');
        cSource.attributes.set('order_by', 'name');
    },
    setInitialState() {
        const mTarget = this.getTargetModel(this.view);

        if (mTarget.get(this.filterKey) === undefined) {
            mTarget.set(this.filterKey, this.cSource.pluck('id'));
        }
    },
    multiselectOptions: {
        selectedText: '',
        noneSelectedText: '',
    },
    multiselectFilterOptions: {
        label: '',
        placeholder: '',
        width: 218,
    },
    stickitSelectOptions: {
        valuePath: 'id',
        labelPath: 'name',
    },
    bindingOptions() {
        const selectOptions = _.extend({
            collection: this.cSource,
        }, this.getOption('stickitSelectOptions'));

        return {
            observe: this.filterKey,
            selectOptions,
            /**
             * {multiselect-hack-321}
             * Hack to update multiselect if an option is added.
             */
            afterUpdate: () => {
                this.ui.selectStickit.multiselect('refresh');
            },
            onGet(val) {
                if (val instanceof Backbone.Collection) {
                    return val.pluck('id');
                }

                return val;
            },
        };
    },
    plugins() {
        return {
            multiselect: {
                bind() {
                    const multiselectOptions = this.getOption('multiselectOptions');
                    const multiselectFilterOptions = this.getOption('multiselectFilterOptions');

                    this.ui.selectStickit.multiselect(multiselectOptions).multiselectfilter(multiselectFilterOptions);
                },
                unbind() {
                    this.ui.selectStickit.multiselect('destroy');
                },
            },
            stickit: {
                bind() {
                    this.view.addBinding(
                        this.getTargetModel(this.view),
                        this._uiBindings.selectStickit,
                        this.bindingOptions()
                    );
                },
                unbind() {

                },
            },
        };
    },
});
