import mPhone from '../model/phone';
import tPhone from './phone.html';

export default Marionette.ItemView.extend({
    durationTimer: null,
    model: mPhone,
    getNumber() {},
    getExtraHeaders() {},
    template: tPhone,
    ui: {
        callButton: '._call',
        hangupButton: '._hangup',
        duration: '._duration',
    },
    events: {
        'click @ui.callButton': 'call',
        'click @ui.hangupButton': 'hangup',
    },
    modelEvents: {
        'change:status': 'renderPhone',
    },
    bindings: {
        ':el': {
            observe: 'status',
            update($el, val) { $el.alterClass('status-*', 'status-' + s.slugify(val)); },
        },
    },
    plugins() {
        return {
            stickit: {
                bind() {
                    this.stickit();
                },
                unbind() {
                    // NOP.
                },
            },
        };
    },
    onDestroy() {
        window.clearInterval(this.durationTimer);
    },
    onRender() {
        this.renderPhone();
    },
    renderPhone() {
        switch (this.model.get('status')) {
            case this.model.constructor.STATUS_READY:
                if (this.durationTimer) {
                    window.clearInterval(this.durationTimer);
                    this.durationTimer = null;
                }

                this.ui.callButton.show();
                this.ui.hangupButton.hide();
                break;
            case this.model.constructor.STATUS_DIALING:
                this.startOfCall = moment();
                this.durationTimer = window.setInterval(() => this.renderDuration(), 1000);
                this.renderDuration();
                this.ui.callButton.hide();
                this.ui.hangupButton.show();
                break;
            case this.model.constructor.STATUS_IN_CALL:
                if (this.durationTimer) {
                    window.clearInterval(this.durationTimer);
                    this.durationTimer = null;
                }
                this.startOfCall = moment();
                this.durationTimer = window.setInterval(() => this.renderDuration(), 1000);
                this.renderDuration();
                this.ui.callButton.hide();
                this.ui.hangupButton.show();
                break;
            default:
                console.log('Unkown phone status ' + this.model.get('status'));
                break;
        }
    },
    renderDuration() {
        const duration = moment.duration(moment().diff(this.startOfCall, 'seconds'), 'seconds');
        this.ui.duration.text(duration.format('m:ss', { trim: false }));
    },
    hangup() {
        this.model.getPhone().phoneCalls.forEach(phoneCall => phoneCall.terminate());
    },
    call() {
        this.model.getPhone().startCall(this.getOption('getNumber')(), {
            extraHeaders: this.getOption('getExtraHeaders')(),
        });
        this.model.setVoiceVolume(1);
    },
});
